use std::io;
use crate::commands::property_control::PropertyControlCommand;
use crate::commands::node_management::NodeManagementCommand;
use crate::packet::raw_packet::RawPacket;

/// Abstraction used for interpreting the data in a packet.
#[derive(Debug, Clone)]
pub enum PacketData {
    NodeManagement(NodeManagementCommand),
    PropertyControl(PropertyControlCommand),
    Raw(Vec<u8>)
}

/// Returns an empty `PacketData::Raw`
impl Default for PacketData {
    fn default() -> Self {
        PacketData::Raw(vec![])
    }
}

impl PacketData {
    pub fn into_bytes(self) -> Vec<u8> {
        match self {
            PacketData::NodeManagement(v) => v.into(),
            PacketData::PropertyControl(v) => v.into(),
            PacketData::Raw(v) => v
        }
    }
}

impl TryFrom<RawPacket> for PacketData {
    type Error = io::Error;
    fn try_from(raw_packet: RawPacket) -> io::Result<Self> {
        match {
            raw_packet.command >> 4
        } {
            0x0 => Ok(PacketData::NodeManagement(NodeManagementCommand::try_from(raw_packet)?)),
            0x1 => Ok(PacketData::PropertyControl(PropertyControlCommand::try_from(raw_packet)?)),
            _ => Err(io::Error::new(io::ErrorKind::InvalidInput, "command group is unsupported"))
        }
    }
}